@@ -2,7 +2,10 @@ adjust_uc_val <- function(
     data_observed,
     data_validation) {
   if (!all(data_observed$confounders %in% data_validation$confounders)) {
-    stop("All confounders in observed data must be present in validation data.")
+    stop(
+      "All confounders in observed data must be present in validation data.",
+      call. = FALSE
+    )
   }
 
   if (
@@ -13,7 +16,8 @@ adjust_uc_val <- function(
         "This function adjusts for unobserved confounding from one confounder.",
         "\n",
         "Validation data must have one more confounder than the observed data."
-      )
+      ),
+      call. = FALSE
     )
   }
 
@@ -223,7 +227,10 @@ adjust_uc_coef <- function(
       )
     }
   } else if (len_c > 3) {
-    stop("This function is currently not compatible with >3 confounders.")
+    stop(
+      "This function is currently not compatible with >3 confounders.",
+      call. = FALSE
+    )
   }
 
   return(final)
@@ -315,7 +322,10 @@ adjust_uc <- function(
     (!is.null(data_validation) && !is.null(u_model_coefs)) ||
       (is.null(data_validation) && is.null(u_model_coefs))
   ) {
-    stop("One of data_validation or u_model_coefs must be non-null.")
+    stop(
+      "One of data_validation or u_model_coefs must be non-null.",
+      call. = FALSE
+    )
   }
   data <- data_observed$data
 
