@@ -1,37 +1,139 @@
-#' Adust for uncontrolled confounding and exposure misclassification.
-#'
-#' @description
-#' `r lifecycle::badge("deprecated")`
-#'
-#' `adjust_uc_emc()` was renamed to `adjust_uc_em()`
-#' @keywords internal
-#'
-#' @export
-adjust_uc_emc <- function(
+adjust_uc_em_val <- function(
     data_observed,
-    u_model_coefs = NULL,
-    x_model_coefs = NULL,
-    x1u0_model_coefs = NULL,
-    x0u1_model_coefs = NULL,
-    x1u1_model_coefs = NULL,
-    level = 0.95) {
-  lifecycle::deprecate_warn(
-    "1.5.3", "adjust_uc_emc()", "adjust_uc_em()"
+    data_validation) {
+  if (!all(data_observed$confounders %in% data_validation$confounders)) {
+    stop(
+      "All confounders in observed data must be present in validation data.",
+      call. = FALSE
+    )
+  }
+
+  if (
+    length(data_validation$confounders) - length(data_observed$confounders) != 1
+  ) {
+    stop(
+      paste0(
+        "This function adjusts for unobserved confounding from one confounder.",
+        "\n",
+        "Validation data must have one more confounder than the observed data."
+      ),
+      call. = FALSE
+    )
+  }
+  if (is.null(data_validation$misclassified_exposure)) {
+    stop(
+      paste0(
+        "This function is adjusting for a misclassified exposure.",
+        "\n",
+        "Validation data must have a true and misclassified exposure specified."
+      ),
+      call. = FALSE
+    )
+  }
+
+  n <- nrow(data_observed$data)
+
+  df <- data.frame(
+    Xstar = data_observed$data[, data_observed$exposure],
+    Y = data_observed$data[, data_observed$outcome]
   )
-  adjust_uc_em(
-    data_observed,
-    u_model_coefs,
-    x_model_coefs,
-    x1u0_model_coefs,
-    x0u1_model_coefs,
-    x1u1_model_coefs,
-    level
+  df <- bind_cols(
+    df,
+    data_observed$data %>%
+      select(all_of(data_observed$confounders))
   )
+
+  if (all(df$Y %in% 0:1)) {
+    y_binary <- TRUE
+  } else {
+    y_binary <- FALSE
+  }
+
+  df_val <- data.frame(
+    X = data_validation$data[, data_validation$true_exposure],
+    Y = data_validation$data[, data_validation$true_outcome],
+    Xstar = data_validation$data[, data_validation$misclassified_exposure]
+  )
+
+  uc <- setdiff(data_validation$confounders, data_observed$confounders)
+  df_val$U <- data_validation$data[, uc]
+  df_val <- bind_cols(
+    df_val,
+    data_validation$data %>%
+      select(all_of(data_observed$confounders))
+  )
+
+  force_match(
+    df$Y,
+    df_val$Y,
+    "Outcomes from both datasets must both be binary or both be continuous."
+  )
+  force_binary(
+    df_val$U,
+    "Uncontrolled confounder in validation data must be a binary integer."
+  )
+  force_binary(
+    df$Xstar,
+    "Exposure in observed data must be a binary integer."
+  )
+  force_binary(
+    df_val$Xstar,
+    "Misclassified exposure in validation data must be a binary integer."
+  )
+  force_binary(
+    df_val$X,
+    "True exposure in validation data must be a binary integer."
+  )
+
+  x_mod <- glm(X ~ Xstar + Y + . - U,
+    family = binomial(link = "logit"),
+    data = df_val
+  )
+
+  x_mod_coefs <- coef(x_mod)
+  x_pred <- x_mod_coefs[1]
+
+  for (i in 2:length(x_mod_coefs)) {
+    var_name <- names(x_mod_coefs)[i]
+    x_pred <- x_pred + df[[var_name]] * x_mod_coefs[i]
+  }
+
+  df$Xpred <- rbinom(n, 1, plogis(x_pred))
+
+  u_mod <- glm(U ~ X + Y,
+    family = binomial(link = "logit"),
+    data = df_val
+  )
+
+  u_mod_coefs <- coef(u_mod)
+  u_pred <- u_mod_coefs[1]
+
+  for (i in 2:length(u_mod_coefs)) {
+    var_name <- names(u_mod_coefs)[i]
+    var_name <- gsub("X", "Xpred", var_name) # col X is not in df
+    u_pred <- u_pred + df[[var_name]] * u_mod_coefs[i]
+  }
+
+  df$Upred <- rbinom(n, 1, plogis(u_pred))
+
+  if (y_binary) {
+    final <- glm(
+      Y ~ Xpred + Upred + . - Xstar,
+      family = binomial(link = "logit"),
+      data = df
+    )
+  } else {
+    final <- lm(
+      Y ~ Xpred + Upred + . - Xstar,
+      data = df
+    )
+  }
+
+  return(final)
 }
 
-# bias adjust with u_model_coefs and x_model_coefs
 
-uc_em_single <- function(
+adjust_uc_em_coef_single <- function(
     data_observed,
     u_model_coefs,
     x_model_coefs) {
@@ -42,6 +144,10 @@ uc_em_single <- function(
   len_u_coefs <- length(u_model_coefs)
   len_x_coefs <- length(x_model_coefs)
 
+  xstar <- data[, data_observed$exposure]
+  y <- data[, data_observed$outcome]
+
+  force_binary(xstar, "Exposure must be a binary integer.")
   force_len(
     len_u_coefs,
     3,
@@ -59,9 +165,6 @@ uc_em_single <- function(
     )
   )
 
-  xstar <- data[, data_observed$exposure]
-  y <- data[, data_observed$outcome]
-
   if (all(y %in% 0:1)) {
     y_binary <- TRUE
   } else {
@@ -187,16 +290,17 @@ uc_em_single <- function(
       )
     }
   } else if (len_c > 3) {
-    stop("This function is currently not compatible with >3 confounders.")
+    stop(
+      "This function is currently not compatible with >3 confounders.",
+      call. = FALSE
+    )
   }
 
   return(final)
 }
 
 
-# bias adjust with multinomial coefs
-
-uc_em_multinom <- function(
+adjust_uc_em_coef_multinom <- function(
     data_observed,
     x1u0_model_coefs,
     x0u1_model_coefs,
@@ -209,6 +313,10 @@ uc_em_multinom <- function(
   len_x0u1_coefs <- length(x0u1_model_coefs)
   len_x1u1_coefs <- length(x1u1_model_coefs)
 
+  xstar <- data[, data_observed$exposure]
+  y <- data[, data_observed$outcome]
+
+  force_binary(xstar, "Exposure must be a binary integer.")
   force_len(
     len_x1u0_coefs,
     3 + len_c,
@@ -234,9 +342,6 @@ uc_em_multinom <- function(
     )
   )
 
-  xstar <- data[, data_observed$exposure]
-  y <- data[, data_observed$outcome]
-
   if (all(y %in% 0:1)) {
     y_binary <- TRUE
   } else {
@@ -529,23 +634,62 @@ uc_em_multinom <- function(
       })
     }
   } else if (len_c > 3) {
-    stop("This function is currently not compatible with >3 confounders.")
+    stop(
+      "This function is currently not compatible with >3 confounders.",
+      call. = FALSE
+    )
   }
 
   return(final)
 }
 
+
+#' Adust for uncontrolled confounding and exposure misclassification.
+#'
+#' @description
+#' `r lifecycle::badge("deprecated")`
+#'
+#' `adjust_uc_emc()` was renamed to `adjust_uc_em()`
+#' @keywords internal
+#'
+#' @export
+adjust_uc_emc <- function(
+    data_observed,
+    u_model_coefs = NULL,
+    x_model_coefs = NULL,
+    x1u0_model_coefs = NULL,
+    x0u1_model_coefs = NULL,
+    x1u1_model_coefs = NULL,
+    level = 0.95) {
+  lifecycle::deprecate_warn(
+    "1.5.3", "adjust_uc_emc()", "adjust_uc_em()"
+  )
+  adjust_uc_em(
+    data_observed,
+    u_model_coefs,
+    x_model_coefs,
+    x1u0_model_coefs,
+    x0u1_model_coefs,
+    x1u1_model_coefs,
+    level
+  )
+}
+
+
 #' Adust for uncontrolled confounding and exposure misclassification.
 #'
 #' `adjust_uc_em` returns the exposure-outcome odds ratio and confidence
 #' interval, adjusted for uncontrolled confounding and exposure
-#' misclassificaiton. Two different options for the bias parameters are
-#' available here: 1) parameters from separate models of *U* and *X*
+#' misclassificaiton.
+#'
+#' Bias adjustment can be performed by inputting either a validation dataset or
+#' the necessary bias parameters. Two different options for the bias parameters
+#' are available here: 1) parameters from separate models of *U* and *X*
 #' (`u_model_coefs` and `x_model_coefs`) or 2) parameters from a
 #' joint model of *U* and *X* (`x1u0_model_coefs`,
 #' `x0u1_model_coefs`, and `x1u1_model_coefs`).
 #'
-#' Values for the regression coefficients can be applied as
+#' Values for the bias parameters can be applied as
 #' fixed values or as single draws from a probability
 #' distribution (ex: `rnorm(1, mean = 2, sd = 1)`). The latter has
 #' the advantage of allowing the researcher to capture the uncertainty
@@ -557,6 +701,12 @@ uc_em_multinom <- function(
 #'
 #' @param data_observed Object of class `data_observed` corresponding to the
 #' data to perform bias analysis on.
+#' @param data_validation Object of class `data_validation` corresponding to
+#' the validation data used to adjust for bias in the observed data. Here, the
+#' validation data should have data for the same variables as in the observed
+#' data, plus data for the true and misclassified exposure corresponding to the
+#' observed exposure in `data_observed`.
+#' There should also be data for the confounder missing in `data_observed`.
 #' @param u_model_coefs The regression coefficients corresponding to the model:
 #' \ifelse{html}{\out{logit(P(U=1)) = &alpha;<sub>0</sub> + &alpha;<sub>1</sub>X + &alpha;<sub>2</sub>Y, }}{\eqn{logit(P(U=1)) = \alpha_0 + \alpha_1 X + \alpha_2 Y, }}
 #' where *U* is the binary unmeasured confounder, *X* is the
@@ -598,22 +748,37 @@ uc_em_multinom <- function(
 #' confidence interval as the vector: (lower bound, upper bound).
 #'
 #' @examples
-#' df <- data_observed(
+#' df_observed <- data_observed(
 #'   data = df_uc_em,
 #'   exposure = "Xstar",
 #'   outcome = "Y",
 #'   confounders = "C1"
 #' )
+#'
+#' # Using validation data -----------------------------------------------------
+#' df_validation <- data_validation(
+#'   data = df_uc_em_source,
+#'   true_exposure = "X",
+#'   true_outcome = "Y",
+#'   confounders = c("C1", "U"),
+#'   misclassified_exposure = "Xstar",
+#' )
+#'
+#' adjust_uc_em(
+#'   data_observed = df_observed,
+#'   data_validation = df_validation
+#' )
+#'
 #' # Using u_model_coefs and x_model_coefs -------------------------------------
 #' adjust_uc_em(
-#'   df,
+#'   data_observed = df_observed,
 #'   u_model_coefs = c(-0.23, 0.63, 0.66),
 #'   x_model_coefs = c(-2.47, 1.62, 0.73, 0.32)
 #' )
 #'
 #' # Using x1u0_model_coefs, x0u1_model_coefs, x1u1_model_coefs ----------------
 #' adjust_uc_em(
-#'   df,
+#'   data_observed = df_observed,
 #'   x1u0_model_coefs = c(-2.82, 1.62, 0.68, -0.06),
 #'   x0u1_model_coefs = c(-0.20, 0.00, 0.68, -0.05),
 #'   x1u1_model_coefs = c(-2.36, 1.62, 1.29, 0.27)
@@ -632,47 +797,42 @@ uc_em_multinom <- function(
 
 adjust_uc_em <- function(
     data_observed,
+    data_validation = NULL,
     u_model_coefs = NULL,
     x_model_coefs = NULL,
     x1u0_model_coefs = NULL,
     x0u1_model_coefs = NULL,
     x1u1_model_coefs = NULL,
     level = 0.95) {
+  check_inputs3(
+    data_validation,
+    list(u_model_coefs, x_model_coefs),
+    list(x1u0_model_coefs, x0u1_model_coefs, x1u1_model_coefs)
+  )
+
   data <- data_observed$data
   xstar <- data[, data_observed$exposure]
   y <- data[, data_observed$outcome]
 
-  force_binary(xstar, "Exposure must be a binary integer.")
-
   if (all(y %in% 0:1)) {
     y_binary <- TRUE
   } else {
     y_binary <- FALSE
   }
 
-  # check that user correctly specified bias parameters
-  if (
-    !(
-      (is.null(u_model_coefs) && is.null(x_model_coefs)) ||
-        ((is.null(x1u0_model_coefs) && is.null(x0u1_model_coefs) &&
-          is.null(x1u1_model_coefs)))
+  if (!is.null(data_validation)) {
+    final <- adjust_uc_em_val(
+      data_observed,
+      data_validation
     )
-  ) {
-    stop(
-      "Bias parameters must be specified for:\n
-       1) u_model_coefs and x_model_coefs OR\n
-       2) x1u0_model_coefs, x0u1_model_coefs, and x1u1_model_coefs."
-    )
-  }
-
-  if (!is.null(x_model_coefs)) {
-    final <- uc_em_single(
+  } else if (!is.null(x_model_coefs)) {
+    final <- adjust_uc_em_coef_single(
       data_observed = data_observed,
       u_model_coefs = u_model_coefs,
       x_model_coefs = x_model_coefs
     )
   } else if (!is.null(x1u0_model_coefs)) {
-    final <- uc_em_multinom(
+    final <- adjust_uc_em_coef_multinom(
       data_observed = data_observed,
       x1u0_model_coefs = x1u0_model_coefs,
       x0u1_model_coefs = x0u1_model_coefs,
